package com.sun.jndi.ldap;

import javax.naming.NameClassPair;
import javax.naming.spi.DirectoryManager;
import javax.naming.CompositeName;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import javax.naming.ldap.Control;
import java.util.Vector;
import javax.naming.directory.Attributes;
import javax.naming.NamingException;
import java.security.AccessController;
import com.sun.jndi.toolkit.ctx.Continuation;
import javax.naming.Name;
import java.security.AccessControlContext;
import javax.naming.Binding;

final class LdapBindingEnumeration extends AbstractLdapNamingEnumeration<Binding>
{
    private final AccessControlContext acc;
    
    LdapBindingEnumeration(LdapCtx ldapCtx, LdapResult ldapResult, Name name, Continuation continuation) throws NamingException {
        super(ldapCtx, ldapResult, name, continuation);
        this.acc = AccessController.getContext();
    }
    
    @Override
    protected Binding createItem(String nameInNamespace, final Attributes attrs, Vector<Control> vector) throws NamingException {
        Object doPrivileged = null;
        String atom = this.getAtom(nameInNamespace);
        if (attrs.get(Obj.JAVA_ATTRIBUTES[2]) != null) {
            try {
                doPrivileged = AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
                    @Override
                    public Object run() throws NamingException {
                        return Obj.decodeObject(attrs);
                    }
                }, this.acc);
            }
            catch (PrivilegedActionException ex) {
                throw (NamingException)ex.getException();
            }
        }
        if (doPrivileged == null) {
            doPrivileged = new LdapCtx(this.homeCtx, nameInNamespace);
        }
        CompositeName name = new CompositeName();
        name.add(atom);
        Object objectInstance;
        try {
            objectInstance = DirectoryManager.getObjectInstance(doPrivileged, name, this.homeCtx, this.homeCtx.envprops, attrs);
        }
        catch (NamingException ex2) {
            throw ex2;
        }
        catch (Exception rootCause) {
            NamingException o = new NamingException("problem generating object using object factory");
            o.setRootCause(rootCause);
            throw o;
        }
        Object o2;
        if (vector != null) {
            o2 = new BindingWithControls(name.toString(), objectInstance, this.homeCtx.convertControls(vector));
        }
        else {
            o2 = new Binding(name.toString(), objectInstance);
        }
        o2.setNameInNamespace(nameInNamespace);
        return o2;
    }
    
    @Override
    protected AbstractLdapNamingEnumeration<? extends NameClassPair> getReferredResults(LdapReferralContext ldapReferralContext) throws NamingException {
        return (AbstractLdapNamingEnumeration)ldapReferralContext.listBindings(this.listArg);
    }
}
